%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
%
% SIMULATES A MODEL WITH TWO OBCS AND PRINTS IRFS
% 
%  OBCs - 1. Borrowing constraint; 
%         2. Macroprudential response depening on credit growth regime
%
%  Can be used to study:
%   1. an unanticipated shock
%   2. a news shock
%   3. an unrealized news shock
%
%  under the following scenarios
%   1. Symmetric rule
%   2. Asymmetric rule
%
% Uses: OCCBIN (Guerrieri and Iacoviello, 2015)
% 
% William Gatt
% 26 February 2018; 11 July 2018
%
% UPDATE FOR JMCB SUBMSSION
%  - Updated OccBin files from Johannes Pfeifer (June 2021)
%  - Using Dynare v4.6.4 
% ---------------------------------------------------------------------------------

clc;
clear all; 

addpath('functions');
addpath('functions/toolkit_files')
addpath('C:\dynare\4.6.4\matlab');

my_colours1;

save_dir = '../Figures';

set(groot,'defaultlinelinewidth',1);
set(groot,'defaultAxesXGrid','on');
set(groot,'defaultAxesYGrid','on');
set(groot, 'defaultAxesTickLabelInterpreter','latex')
set(groot, 'defaultLegendInterpreter','latex');
set(groot, 'defaultTextInterpreter','latex')

% Choose whether to:
    printtt     = 0; % save figure as PDF
    del_dyn_file= 0; % deletes auxiliary files
    DynParse1   = 0; % ALWAYS SET TO 0 FOR THIS CODE!
    load_noLTV  = 1; % Loads IRFs when there is no policy active (have to regenerate these if any parameters (other than delta_m) are changed in 'param_calib' using 'run_borrconst.m')
    illustrate  = 1; % Loads IRFs used for Figure 1, which gives a flavour to how the rule works and its effect on observables (parameters are changed in 'param_calib')
    
% Pick innovation for IRFs
    irfshock        = char('eps_ETA','eps_ETA_surprise','eps_j4','eps_j');  % bubble shock (shocks 1 and 2), news shock (shock 3), unanticipated shock (shock 4)
    [nshocks,yyy]   = size(irfshock);
    irfperiods      = 25;
    nperiods        = irfperiods+200;
    shockssequence  = zeros(nperiods,nshocks);     
    maxiterr        = 100;
    
% Calibrating shock size and type
    shockstd    = [0.054];  % 1 standard deviation shock --> 1% change in q
    shockON     = [1  1 ... % 1 = unrealised news shock
                      0 ... % 1 = news shock
                      0 ];  % 1 = unanticipated shock
    sudden_collapse = 1;    % Bubble collapses immediately
    shock_sign  = +1;       % Positive or negative initial shock (+/-)
    shock_size  = 2.0;      % Multiplier on standard deviation
    shock_time  = 5:8;      % Shock starts at period 5 and persists for 4 periods 
    news_dur    = max(4,shock_time(end)-shock_time(1)+1);        % Duration of news

    
%% CREATE SHOCK SEQUENCE
if sudden_collapse==1 && sum(shockON(1:2))==2
    shockssequence(shock_time,1)    =  shockstd*shockON(1)*shock_sign*shock_size;           % Bubble shock - boom
    shockssequence(shock_time(1)+news_dur,2)  =  -sum(shockssequence(shock_time,1));        % Bubble shock - IMMEDIATE bust
    shockssequence(shock_time,3)    =  shockstd*shockON(3)*shock_sign*shock_size;           % News shock
    shockssequence(shock_time,4)    =  shockstd*shockON(4)*shock_sign*shock_size;           % Unanticipated shock
else
    shockssequence(shock_time,1)    =  shockstd*shockON(1)*shock_sign*shock_size;           % Bubble shock - boom
    shockssequence(shock_time+news_dur,2)  =  shockstd*shockON(2)*(-shock_sign)*shock_size; % Bubble shock - bust
    shockssequence(shock_time,3)    =  shockstd*shockON(3)*shock_sign*shock_size;           % News shock
    shockssequence(shock_time,4)    =  shockstd*shockON(4)*shock_sign*shock_size;           % Unanticipated shock
end

tic;

%% RUN MODEL
% SYMMETRIC MACROPRUDENTIAL RULE, BORROWING CONSTRAINT DOES NOT BIND
    run_borrconst;

% Extract and save IRFS in % dev from steady state
    IRFS_sym(:,1)      = Y_p(1:irfperiods, 1)*100./Y_ss;
    IRFS_sym(:,2)      = PI4_p(1:irfperiods, 1)*100;
    IRFS_sym(:,3)      = (B_4Y_p(1:irfperiods, 1) + B_4Y_ss)*100;
    IRFS_sym(:,4)      = q_p(1:irfperiods, 1)*100./q_ss;
    IRFS_sym(:,5)      = r4_p(1:irfperiods, 1) + r4_ss;
    IRFS_sym(:,6)      = m_p(1:irfperiods, 1)*100./m_ss;
    IRFS_sym(:,7)      = Cb_p(1:irfperiods, 1)*100./Cb_ss;
    IRFS_sym(:,8)      = Cs_p(1:irfperiods, 1)*100./Cs_ss;
    IRFS_sym(:,9)      = Hb_p(1:irfperiods, 1)*100./Hb_ss;
    IRFS_sym(:,10)     = Nb_p(1:irfperiods, 1)*100./Nb_ss;
    IRFS_sym(:,11)     = mju_p(1:irfperiods,1)+mju_ss;
    IRFS_sym(:,12)     = j_p(1:irfperiods, 1)*100./j_ss;  % housing pref cannot be observed when there is a 'bubble' shock, 
    Sym_mju_p = mju_p;

    save('IRFS_sym.mat','IRFS_sym','Sym_mju_p')

% Uncomment to generate IRFS for passive LTV (although these already exist in the repository)
%     noLTV_mju_p = mju_p;
%     IRFS_noLTV = IRFS_sym;
%     if shock_sign > 0
%         save('IRFS_noLTV_pos.mat','IRFS_noLTV','noLTV_mju_p')
%     else
%         save('IRFS_noLTV_neg.mat','IRFS_noLTV','noLTV_mju_p')
%     end
%     return

    
% ASYMMETRIC MACROPRUDENTIAL RULE, BORROWING CONSTRAINT DOES NOT BIND
    run_borrconst_asy;

% Extract and save IRFS in % dev from steady state
    IRFS_asy(:,1)      = Y_p(1:irfperiods, 1)*100./Y_ss;
    IRFS_asy(:,2)      = PI4_p(1:irfperiods, 1)*100 ;
    IRFS_asy(:,3)      = (B_4Y_p(1:irfperiods, 1) + B_4Y_ss)*100;
    IRFS_asy(:,4)      = q_p(1:irfperiods, 1)*100./q_ss;
    IRFS_asy(:,5)      = r4_p(1:irfperiods, 1)+r4_ss;
    IRFS_asy(:,6)      = m_p(1:irfperiods, 1)*100./m_ss;
    IRFS_asy(:,7)      = Cb_p(1:irfperiods, 1)*100./Cb_ss;
    IRFS_asy(:,8)      = Cs_p(1:irfperiods, 1)*100./Cs_ss;
    IRFS_asy(:,9)      = Hb_p(1:irfperiods, 1)*100./Hb_ss;
    IRFS_asy(:,10)     = Nb_p(1:irfperiods, 1)*100./Nb_ss;
    IRFS_asy(:,11)     = mju_p(1:irfperiods,1)+mju_ss;
    IRFS_asy(:,12)     = j_p(1:irfperiods, 1)*100./j_ss;  % housing pref cannot be observed when there is a 'bubble' shock, 

    Asym_mju_p = mju_p;

    save('IRFS_asy.mat','IRFS_asy')

% Load IRFs from symmetric regime
    load IRFS_sym 
    if load_noLTV
        if shock_sign > 0
            load IRFS_noLTV_pos
        else
            load IRFS_noLTV_neg
        end
    end

% Print info about Lagrange multiplier and simulation time
     n_slack_sym = sum(Sym_mju_p/(-mju_ss)>0.9999);
     n_slack_asy = sum(Asym_mju_p/(-mju_ss)>0.9999);
     if load_noLTV
         n_slack_noLTV = sum(noLTV_mju_p/(-mju_ss)>0.9999);
     end
        
    fprintf('\nBorrowing constraint is slack in: \n')
    fprintf(' %2.0f quarter(s) in Passive regime \n', n_slack_noLTV);
    fprintf(' %2.0f quarter(s) in Symmetric regime \n', n_slack_sym);
    fprintf(' %2.0f quarter(s) in Asymmetric regime\n', n_slack_asy);  %when constraint is slack, mju_p is *almost* equal to -mju_ss
    etime=toc;
    fprintf('\nTotal simulation time: %4.2f seconds\n', etime);

    str1 = "Borrowing constraint is slack in:";
    str2 = " quarters in Passive regime";
    str3 = " quarters in Symmetric regime";
    str4 = " quarter in Asymmetric regime";

%     OBC_message = str1 + ...
%                  newline + ...
%                  "  " + n_slack_noLTV + str2 +...                 
%                 newline + ...
%                  "  " + n_slack_sym + str3 +...
%                  newline + ...
%                  "  " + n_slack_asy + str4;

 % Storing periods in which borrowing constraint is slack
  slack_pos_sym = [find(IRFS_sym(:,11)<0.0001)];
  slack_pos_asy = [find(IRFS_asy(:,11)<0.0001)];
  if load_noLTV
      slack_pos_noLTV = [find(IRFS_noLTV(:,11)<0.0001)];
  end
  
  if illustrate     % Runs IRFs with arbitrary calibration for asymmetry (2 x) to create Figure 1
%       if shock_sign > 0
%           save IRFs_Fig1.mat      
%       else
%           save IRFs_Fig1_neg.mat      
%       end
  end
      
  %     % Clear dynare files that are not needed
            if del_dyn_file == 1
                filenames = [   
                                "Mod_borrconst_binding_sym";
                                "Mod_borrconst_slack_sym";
                                "Mod_borrconst_binding_low";
                                "Mod_borrconst_binding_high";
                                "Mod_borrconst_slack_low";
                                "Mod_borrconst_slack_high"];
                for i=1:length(filenames)
                    delete(strcat(filenames(i),'.m'),strcat(filenames(i),'_dynamic.m'),strcat(filenames(i),'_static.m'),strcat(filenames(i),'_set_auxiliary_variables.m'));
                end
            end
              
            
%%  CREATE PLOTS
fig_name = ['FIGURE'];

set(0,'DefaultLineLineWidth',2);
%% ILLUSTRATIVE FIGURE (FIGURE 1)

if load_noLTV & illustrate
    if shock_sign > 0
        load IRFS_Fig1
    else
        load IRFS_Fig1_neg
    end
    
    figure('Name',fig_name,'PaperPositionMode','auto',...
                 'PaperOrientation','portrait',...
                  'PaperSize',[30 25],'Color','white')
%                     'PaperSize',[35 12],'Color','white')
            

    nper = size(IRFS_sym(:,1),1);
    zeroline = zeros(irfperiods,1)';
    timeline=[-4:irfperiods-5]';

    %---HOUSE PRICES---
    subplot(2,2,1); hold on; grid off; box on;
    h1=plot(timeline,IRFS_sym(:, 4), 'color', red,'Linestyle','-.','Linewidth',2.5);%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    h3=plot(timeline,IRFS_asy(:, 4),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,IRFS_noLTV(:, 4),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,zeroline,'Color',grey,'LineStyle','-','Linewidth',1.0)
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 4);   
    hh7=plot(timeline(slack_pos_noLTV), IRFS_noLTV(slack_pos_noLTV, 4),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',5,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    ylabel('\% dev. from SS')
    xlim([-4 irfperiods-4-1]); 
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    ytickformat('%2.1f')
    title('House prices','FontSize',18); grid off; %axis tight;
    
    %---OUTPUT---
    subplot(2,2,3); hold on; grid off; box on;
    h1=plot(timeline,IRFS_sym(:, 1), 'color', red,'Linestyle','-.','Linewidth',2.5);%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    h3=plot(timeline,IRFS_asy(:, 1),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,IRFS_noLTV(:, 1),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,zeroline,'Color',grey,'LineStyle','-','Linewidth',1.0)
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 1);   
    hh7=plot(timeline(slack_pos_noLTV), IRFS_noLTV(slack_pos_noLTV,1),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',5,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    ylabel('\% dev. from SS')
    xlim([-4 irfperiods-4-1]); 
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    ytickformat('%2.1f')
    title('Output','FontSize',18); grid off; %axis tight;
    
    % Legend
        kappa = (delta_mH/delta_mL);
        hL=legend([h4,h1,h3,hh7],{'Fixed LTV','Symmetric rule','Asymmetric rule','Slack constraint'},...
            'FontSize',16,'Orientation','horizontal');
            newUnits = 'normalized';
            newPosition = [0.495 -0.01 0.02 0.08];
        set(hL,'Position', newPosition,'Units', newUnits,'box', 'off');

    
    %---CREDIT TO GDP RATIO---
    subplot(2,2,2); hold on; grid off; box on
    h1=plot(timeline,IRFS_sym(:, 3),  'color',red,'Linestyle','-.','Linewidth',2.5);%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    h3=plot(timeline,IRFS_asy(:, 3),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,IRFS_noLTV(:, 3),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,B_4Y_ss*100*ones(irfperiods,1),'LineWidth',0.75,'Color',grey,'LineStyle','-');
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 3)   ;
    hh7=plot(timeline(slack_pos_noLTV,1), IRFS_noLTV(slack_pos_noLTV,3),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',5,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    ylabel('Level')
    xlim([-4 irfperiods-4-1]); 
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    title('Credit to output ratio (\%)','FontSize',18); grid off; %axis tight;
    
    %---LTV RATIO---
    subplot(2,2,4); hold on; grid off; box on;
    h1=plot(timeline,(1+IRFS_sym(:, 6)/100)*0.9, 'color',red,'Linestyle','-.','Linewidth',2.5);%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    h3=plot(timeline,(1+IRFS_asy(:, 6)/100)*0.9,'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,(1+IRFS_noLTV(:, 6)/100)*0.9,'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,m_ss*ones(irfperiods,1),'LineWidth',0.75,'Color',grey,'LineStyle','-');
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, (1+IRFS_sym/100)*0.9, (1+IRFS_asy/100)*0.9, slack_pos_sym, slack_pos_asy, 6)   ;
    hh7=plot(timeline(slack_pos_noLTV,1), (1+IRFS_noLTV(slack_pos_noLTV,6)/100)*0.9,'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',5,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    title('LTV ratio','FontSize',12); grid off; %axis tight;
    ylabel('Level')
    xlim([-4 irfperiods-4-1]);
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    ytickformat('%2.2f')
    title('LTV ratio','FontSize',18); grid off; %axis tight;

            
    printtt = 0 ; %switch on to save
    if  printtt==1
        if shock_sign > 0
            print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_boom_bust_FIG1.pdf'));
        else
            print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_boom_bust_FIG1_neg.pdf'));
        end
    end
end
   

%% FULL FIGURE (FIGURE 6)
if load_noLTV == 1 & illustrate == 0
    figure('Name',fig_name,'PaperPositionMode','auto',...
                 'PaperOrientation','portrait',...
                  'PaperSize',[27 20],'Color','white')
            % 'PaperSize',[11 7],'Color','white')    

    nper = size(IRFS_sym(:,1),1);
    zeroline = zeros(irfperiods,1)';
    timeline=[-4:irfperiods-5]';

    subplot(2,3,1); hold on; grid off; box on;
    h1=plot(timeline,IRFS_sym(:, 1), 'color', red,'Linestyle','-.','Linewidth',2.5);%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    h3=plot(timeline,IRFS_asy(:, 1),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,IRFS_noLTV(:, 1),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,zeroline,'Color',grey,'LineStyle','-','Linewidth',1.0)
    if shock_sign < 0
        ylim([-2 0.75])
    end
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 1);   
    hh7=plot(timeline(slack_pos_noLTV), IRFS_noLTV(slack_pos_noLTV,1),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',4,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    title('Output','FontSize',12); grid off; %axis tight;
    ylabel('\% dev. from SS')
    xlim([-4 irfperiods-4-1]); 
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    ytickformat('%2.1f')


    % LEGEND
    kappa = (delta_mH/delta_mL);
    hL=legend([h4,h1,h3,hh7],{'Fixed LTV','Symmetric rule','Asymmetric rule','Slack constraint'},...
        'FontSize',14,'Orientation','horizontal');
        newUnits = 'normalized';
        newPosition = [0.495 0.00 0.05 0.08];
    set(hL,'Position', newPosition,'Units', newUnits,'box', 'off');

    subplot(2,3,2); hold on; grid off; box on;
    plot(timeline,IRFS_sym(:, 2), 'color',red,'Linestyle','-.','Linewidth',2.5)%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    plot(timeline,IRFS_asy(:, 2),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,IRFS_noLTV(:, 2),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,zeroline,'Color',grey,'LineStyle','-','Linewidth',1.0)
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 2) ;  
    hh7=plot(timeline(slack_pos_noLTV,1), IRFS_noLTV(slack_pos_noLTV,2),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',4,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    title('Inflation','FontSize',12); grid off; 
    ylabel('Ppts.')
    xlim([-4 irfperiods-4-1]); 
    xticks(-4:4:irfperiods-4)
    if shock_sign > 0
        yticks([-0.8:0.20:1.0])
    else
        yticks([-1.8: 0.40 : 1.8])
    end
    ylim([min(yticks) max(yticks)])
    set(gca,'FontSize',12)
    ytickformat('%2.1f')



    subplot(2,3,3); hold on; grid off; box on
    h1=plot(timeline,IRFS_sym(:, 3),  'color',red,'Linestyle','-.','Linewidth',2.5);%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    h3=plot(timeline,IRFS_asy(:, 3),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,IRFS_noLTV(:, 3),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,B_4Y_ss*100*ones(irfperiods,1),'LineWidth',0.75,'Color',grey,'LineStyle','-');
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 3)   ;
    hh7=plot(timeline(slack_pos_noLTV,1), IRFS_noLTV(slack_pos_noLTV,3),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',4,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    title('Credit to output ratio (\%)','FontSize',12); grid off; %axis tight;
    ylabel('Level')
    xlim([-4 irfperiods-4-1]); 
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    

    subplot(2,3,4); hold on; grid off; box on
    plot(timeline,IRFS_sym(:,5), 'color', red,'Linestyle','-.','Linewidth',2.5)%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    plot(timeline,IRFS_asy(:, 5),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,r4_ss*ones(irfperiods,1),'LineWidth',0.75,'Color',grey,'LineStyle','-');
    h4=plot(timeline,IRFS_noLTV(:, 5),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 5)   ;
    hh7=plot(timeline(slack_pos_noLTV,1), IRFS_noLTV(slack_pos_noLTV,5),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',4,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    title('Nom. interest rate (ann. \%)','FontSize',12); grid off; %axis tight;
    ylabel('Level')
    xlim([-4 irfperiods-4-1]); 
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    ytickformat('%2.1f')
    
    IRFS_sym1(:,6) = (1+IRFS_sym(:, 6)/100)*0.9;
    IRFS_asy1(:,6) = (1+IRFS_asy(:, 6)/100)*0.9;
    
    
    subplot(2,3,5); hold on; grid off; box on;
    h1=plot(timeline,(1+IRFS_sym(:, 6)/100)*0.9, 'color',red,'Linestyle','-.','Linewidth',2.5);%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    h3=plot(timeline,(1+IRFS_asy(:, 6)/100)*0.9,'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    h4=plot(timeline,(1+IRFS_noLTV(:, 6)/100)*0.9,'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,m_ss*ones(irfperiods,1),'LineWidth',0.75,'Color',grey,'LineStyle','-');
    line([0 0],[min(ylim) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    mark_slack_constraint(timeline, (1+IRFS_sym/100)*0.9, (1+IRFS_asy/100)*0.9, slack_pos_sym, slack_pos_asy, 6)   ;
    hh7=plot(timeline(slack_pos_noLTV,1), (1+IRFS_noLTV(slack_pos_noLTV,6)/100)*0.9,'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',4,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    title('LTV ratio','FontSize',12); grid off; %axis tight;
    ylabel('Level')
    xlim([-4 irfperiods-4-1]);
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    ytickformat('%2.2f')
    
    
    subplot(2,3,6); hold on; grid off; box on;
    plot(timeline,IRFS_sym(:, 11),  'color',red,'Linestyle','-.','Linewidth',2.5)%,'Marker','o','MarkerSize',5,'MarkerIndices',1:1:nper,'Markerfacecolor',red); hold on; grid off; box on;;
    plot(timeline,IRFS_asy(:, 11),'Color',lightblue,'LineStyle','-','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    plot(timeline,mju_ss*ones(irfperiods,1),'LineWidth',0.75,'Color',grey,'LineStyle','-');
    h4=plot(timeline,IRFS_noLTV(:, 11),'Color',green,'LineStyle',':','Linewidth',2.5,'Marker','none','MarkerSize',8,'MarkerIndices',1:1:nper);
    line([0 0],[min(min(ylim),-0.01) max(ylim)],'lineStyle','--','linewidth',0.75,'color',grey);
    plot(timeline,zeroline,'Color',grey,'LineStyle','-','Linewidth',1.0)
    mark_slack_constraint(timeline, IRFS_sym, IRFS_asy, slack_pos_sym, slack_pos_asy, 11)   ;
    hh7=plot(timeline(slack_pos_noLTV,1), IRFS_noLTV(slack_pos_noLTV,11),'lineStyle','none','marker','o',...
                                            'linewidth',0.1,'MarkerSize',4,'Color','k',...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
    title('Lagrange multiplier','FontSize',12); grid off; %axis tight;
    ylabel('Level');
    xlim([-4 irfperiods-4-1]); 
%     ylim([-0.000 max(IRFS_sym(:,11))*1.1]);
    xticks(-4:4:irfperiods-4)
    set(gca,'FontSize',12)
    ylim([-0.01 max(ylim)])
    ytickformat('%2.2f')
    %  text(4, mju_ss-0.03, {OBC_message}, 'clipping', 'off','FontSize',8, 'Rotation',0);

%     text(5, mju_ss-0.03, str1 + ...
%                      newline + ...
%                      "  " + n_slack_sym + str2 +...
%                      newline + ...
%                      "  " + n_slack_asy + str3, 'clipping', 'off','FontSize',10, 'Rotation',0);

        if  printtt==1 
            if shock_sign > 0 
                print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_boom_bust.pdf'));
            else
                print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_boom_bust_neg.pdf'));
            end
        end
end
 
 

