%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
%  SIMULATES A MODEL WITH TWO OBCS AND PRINTS IRFS
% 
% 
% William Gatt
% ---------------------------------------------------------------------------------

% REMEMBER: switch on control variable in Mod_borrconst_binding_sym_iterparam.mod and
% Mod_borrconst_slack_sym.mod ( iterate_params = 1 )

clear all; 
warning off

set(0,      'defaultlinelinewidth'              ,1.5)
set(groot,  'defaultAxesXGrid',                 'off')
set(groot,  'defaultAxesYGrid',                 'off')
set(groot,  'defaultAxesTickLabelInterpreter',  'latex')
set(groot,  'defaultLegendInterpreter',         'latex');
set(groot,  'defaultTextInterpreter',           'latex')

addpath('functions/toolkit_files')
addpath('functions')
save_dir = '../Figures/';

% global M_ oo_ tnumperiods popup maxnumreps iter
% global  tnumperiods popup maxnumreps iter
%% SETTINGS

% Load and/or results
    load_results = 0; %loads pre-set results (if they exist)
    save_results = 0; 
    save_fig     = 0;

    warning off
    
if load_results == 0  
% Model regimes
    Set.regime_11       = 'Mod_borrconst_binding_high_iterparam'; % constraint BINDS, Credit BOOM
    Set.regime_01       = 'Mod_borrconst_slack_high_iterparam';   % constraint SLACK, Credit BOOM
    Set.regime_10       = 'Mod_borrconst_binding_low_iterparam';  % constraint BINDS, Credit BUST
    Set.regime_00       = 'Mod_borrconst_slack_low_iterparam';    % constraint SLACK, Credit BUST


% Occasionally binding constraint
    Set.constraint1          =   'mju<-mju_ss';     % escape from reference model - constraint becomes slack
    Set.constraint_relax1    =   'MaxLev > 0';      % back to reference model     - constraint becomes binding
    Set.constraint2          =   'B_4Y < -0.0001';  % escape from reference model - constraint becomes slack
    Set.constraint_relax2    =   'B_4Y > -0.0001';  % back to reference model     - constraint becomes binding
    
    
% Pick innovation for IRFs
    irfshock    =   char('eps_ETA','eps_ETA_surprise','eps_j','eps_j4');  % label for innovation for IRFs    
  [nshocks,yyy] =   size(irfshock);
    nshckperiods=   400; % number of periods in which shock hits (better set this low and numiter high, quicker)
    extraperiods=   100; % extra periods with shocks = 0 to allow model to converge back to SS
    numiter     =   100;  % number of times the economy is simulated (at a given parameterisation)
    shockstd    =   [0.054 0.054 0.054 0.054]; % 1 standard deviation shock --> change in q by 1%
    shock_size  =   1.0;
    maxiterr    =   100; % OCCBIN maximum number of iterations
    suprise     =   1.0; % 0<surprise<1, with 1 --> 100% of news shock false, 0 --> 100% of news shock true
    nperiods    =   nshckperiods + extraperiods;
    
    DynParse1   =   1; % Parse Dynare mod-files only once (be careful) (need to run short simulation once to set up structures)
    print_output=   1;
    
   
    regime_count = zeros(nshocks,numiter);


%     rng(2479,'twister');  % fixed - simulations over different policy rules need the same shocks!
    rng(190519,'twister'); 

    allshocks = zeros(nshckperiods,nshocks*numiter);
    convergence_errs = zeros(nshckperiods,numiter);
    

    % Main grid used

    %---BENCHMARK----
    delta_m_sym = 0.75;
    delta_mHv = [1:1.5:50]';
    
    rho_mv   = [0.0]';
    kink = delta_mHv/delta_m_sym;
    
%   % Accuracy check
    
    num_del = length(delta_mHv);
    num_rho = length(rho_mv);
    num_del*num_rho;
%% SET UP SHOCK MATRICES
    switch numiter
    case 1 
        shockdraws = randn(nshckperiods,nshocks);  % number of periods x number of shocks

        % Create news shocks
        shockdraws(nshckperiods-3:nshckperiods,1) = 0;

        % Create 'surprise' (of shock not occuring)
        shockdraws(:,2) = 0;
        shockdraws(5:nshckperiods,2) = -suprise*shockdraws(1:nshckperiods-4,1);
        
        % Full matrix of shocks
        allshocks = shockdraws.*shockstd.*shock_size; 
    
    otherwise
        shockdraws = randn(nshckperiods,nshocks*numiter);
        %   Create news shocks in 'blocks' of iterations, starting from 
        %   col 1, col 4, col 7, etc (news shock, correction, preference shock)
        
        % Create 'surprise' (of shock not occuring)
        for ii = 1:nshocks:numiter*nshocks-2 
            shockdraws(nshckperiods-3:nshckperiods,ii) = 0;
            shockdraws(:,ii+1) = 0;
            shockdraws(5:nshckperiods,ii+1) = -suprise*shockdraws(1:nshckperiods-4,ii);
        end

        for iter = 1:numiter
            % Full matrix of shocks
            if nshocks>2 
                step = iter*nshocks-(nshocks-1);
                allshocks(:,step:step+nshocks-1) = shockdraws(:,step:step+nshocks-1).*shockstd.*shock_size;
            else
                step = iter*nshocks-1;
                allshocks(:,step:step+1) = shockdraws(:,step:step+1).*shockstd.*shock_size;
            end
        end
    end    

        

counting = 0;


%% MAIN LOOPS
    fprintf('\n======================================================== \n');
    fprintf('Solving model using OCCBIN \n - Model with OBC in borrowing and asymmetric LTV rule \n')
    fprintf('========================================================\n');


    counter = 0;
    total_iters = num_del*num_rho;
    tic;
    
    for del = 1:num_del
        
        for rho = 1:num_rho
            
            delta_mHv1 = delta_mHv(del);
            rho_v1 = rho_mv(rho);
            
            fprintf('\nParameters: delta_mH = %2.2f, rho_m = %2.2f\n',delta_mHv1,rho_v1);
            
            %overwrite default parameters before passing them to OCCBIN
            fileid = fopen('param_calib_sim.mod','w');
            add_params = strcat('delta_mH=',num2str(delta_mHv1),'; rho_m=',num2str(rho_v1),';');
            fprintf(fileid,'%c',add_params);
            
    %% PRE-PARSE MOD FILES IF NEEDED (for speed)
        if DynParse1 == 1 %parse Dynare mod files once at the beginning to speed up loop execution
            fprintf('\n---------------------------------\n')
            fprintf('Pre-parsing MOD files for speed')
            fprintf('\n---------------------------------\n')

            disp('Regime 1')
            eval(['dynare ',Set.regime_11,' noclearall nolog nopreprocessoroutput'])
            oo00_ = oo_;
            M00_ = M_;
            pause(0.1);
            save refmod1 oo00_ M00_
            
            
            disp('Regime 2')
            eval(['dynare ',Set.regime_01,' noclearall nopreprocessoroutput '])
            oo10_ = oo_;
            M10_ = M_;
            pause(0.1);
            save refmod2 oo10_ M10_
            
            
            disp('Regime 3')
            eval(['dynare ',Set.regime_10,' noclearall nopreprocessoroutput '])
            oo01_ = oo_;
            M01_ = M_;
            pause(0.1);
            save refmod3 oo01_ M01_
            

            disp('Regime 4')
            eval(['dynare ',Set.regime_00,' noclearall nopreprocessoroutput'])
            oo11_ = oo_;
            M11_ = M_;
            pause(0.1);
            save refmod4 oo11_ M11_
            fprintf('---------------------------------\n')
        end
        
     %% MAIN LOOP
 
            counter = counter + 1;
            if mod(counter,10)==0
                fprintf('--> Iteration %3.0f of %3.0f\n', counter, total_iters)
            end
            for iter = 1:numiter
                if nshocks>2
                    step = iter*nshocks-(nshocks-1);
                    shockssequence = allshocks(:,step:step+nshocks-1);
                else
                    step = iter*nshocks-1;
                    shockssequence = allshocks(:,step:step+1);
                end
                
               
                [zdatalinear, zdatapiecewise, zdatass, oobase_, Mbase_, conv_errs, reg_count] = ...
                    solve_two_constraints1(...
                    Set.regime_11,Set.regime_01,Set.regime_10,Set.regime_00,...
                    Set.constraint1, Set.constraint2,...
                    Set.constraint_relax1, Set.constraint_relax2,...
                    shockssequence,irfshock,  nperiods,  0,            maxiterr,0,DynParse1);
                    
            
                convergence_errs(:,iter) = conv_errs;
                counting = counting + nshckperiods;
                regime_count(:,iter) = [reg_count];
                
                for i=1:Mbase_.endo_nbr
                    eval([deblank(Mbase_.endo_names{i,:}),'_l(:,iter,del,rho)   =   zdatalinear(:,i);']);
                    eval([deblank(Mbase_.endo_names{i,:}),'_p(:,iter,del,rho)   =   zdatapiecewise(:,i);']);
                    eval([deblank(Mbase_.endo_names{i,:}),'_ss                  =   zdatass(i);']);
                end
            end
            
            mju_simulated = []; % pre-allocate or clear to avoid issues
            mju_simulated = mju_p(1:end-extraperiods,:)+mju_ss;
            num_mju_neg = [sum(mju_simulated <= 1e-8)*100/nshckperiods]'; %number of times mju is virtually 0
            num_q_neg   = sum((sum(q_p(1:end-extraperiods)+q_ss<0)));
            num_r_neg   = sum((sum(r4_p(1:end-extraperiods)+r4_ss<0)));
            
            if print_output == 1
                fprintf('\n------------ OUTPUT ---------------\n')
                fprintf('Number of times:\n');
                fprintf('   Credit boom, constraint binding:    %5.2f%%\n',sum(regime_count(1,:))*100/(nshckperiods*numiter))
                fprintf('   Credit boom, constraint slack:      %5.2f%%\n',sum(regime_count(2,:))*100/(nshckperiods*numiter))
                fprintf('   Credit bust, constraint binding:    %5.2f%%\n',sum(regime_count(3,:))*100/(nshckperiods*numiter))
                fprintf('   Credit bust, constraint slack(!):   %5.2f%%\n',sum(regime_count(4,:))*100/(nshckperiods*numiter))
                fprintf('\n Av. borrowing constraint is slack: %4.1f%% \n', mean(num_mju_neg));
            end           
            if num_q_neg >0 || num_r_neg>0
                fprintf('   q is negative:                  %4d\n',num_q_neg )
                fprintf('   r is negative:                  %4d\n', num_r_neg)
            end
            
            if sum(sum(convergence_errs)) >0
                fprintf('\nCONVERGENCE PROBLEMS in %2d simulations!\n',sum(sum(convergence_errs)));
            end
        end
    end
    etime2=toc;
    fprintf('\nTotal simulation time: %4.4f minutes\n', etime2/60);
    
end

%% CALCULATING AND PLOTTING LOSS

    if load_results
            load("Loss_Kink_2OBCs_big")
    end

    Loss_credit         = NaN(num_del,num_rho);
    Loss_volatility     = NaN(num_del,num_rho);
    Loss_output         = NaN(num_del,num_rho);
    Loss                = NaN(num_del,num_rho);
    kink = delta_mHv/delta_m_sym;
    
    % LOSS FUNCTION
    for del = 1:num_del
        for rho = 1:num_rho
            Loss_credit(del,rho) = mean(var(B_4Y_p(1:nshckperiods,:,del,rho)./B_4Y_ss));
            Loss_volatility(del,rho) = mean(var(m_p(1:nshckperiods,:,del,rho)./m_ss));
            Loss_output(del,rho) = mean(var(Y_p(1:nshckperiods,:,del,rho)));

            %Angelini-Neri-Panetta (2014) assume weight_volatility = 0.1
            %and weight_output = 0
            weight_credit = 1.0;
            if robustness == 0
                weight_volatility = 0.50;
            else
                weight_volatility = 0.25;
            end
            Loss = weight_credit*Loss_credit + weight_volatility*Loss_volatility;
        end
    end
    
    minLoss = min(Loss,[],'all');
    maxLoss = max(Loss,[],'all');
    [rowMin,colMin] = find(Loss==minLoss,2);
    range = exp(linspace(log(minLoss),log(maxLoss),5));

    if save_results
        if robustness == 0
            save Loss_Kink_2OBCs_big oo_ M_ Loss_credit Loss_volatility Loss B_4Y_p B_4Y_ss m_p m_ss Y_p Y_ss q_p q_l q_ss mju_p mju_ss delta_mHv delta_m_sym rho_mv nshckperiods extraperiods numiter shockstd shock_size num_del num_rho
        else
            save Loss_Kink_2OBCs_big_robust oo_ M_ Loss_credit Loss_volatility Loss B_4Y_p B_4Y_ss m_p m_ss Y_p Y_ss q_p q_l q_ss mju_p mju_ss delta_mHv delta_m_sym rho_mv nshckperiods extraperiods numiter shockstd shock_size num_del num_rho
        end
    end
    
    %%
    close all;
    
    kappa = delta_mHv./delta_m_sym;

    f=figure('Name','Optimal delta_m','PaperPositionMode','auto',...
                 'PaperOrientation','portrait',...
                 'PaperSize',[20 15],'Color','white');
             
    plot(kappa,Loss,'LineStyle','-','marker','none'); grid off; hold on;
    plot(kappa(rowMin),minLoss,'marker','o','markeredgecolor','r','markerfacecolor','r','markersize',7)
    xlabel('$\kappa$','interpreter','latex');
    ylabel('Loss');
%     xlim([1 max(kink)])
%     set(gca,'Xtick',[1 10 20 30 40 50 60])
    if robustness == 0    
        ytickformat('%1.1f')
    else
         ytickformat('%2.2f')
    end
    
    fprintf('\n\nOptimal parameter values (shock std: %3.3f) \n   delta_m: %2.4f ==> kappa = %2.1f\n',shockstd(1)*shock_size,delta_mHv(rowMin),kappa(rowMin))
    set(gca,'FontSize',12)
    if robustness ==0
        axes('Position',[.48 .46 .4 .4])
        plot(kappa,Loss','LineStyle','-','marker','none'); grid off; hold on;
        plot(kappa(rowMin),minLoss,'marker','o','markeredgecolor','r','markerfacecolor','r','markersize',7)
        xlabel('$\kappa$','interpreter','latex');
        ylabel('Loss');
        xlim([25 63])
        xlim([0.5*kappa(rowMin) 1.5*round(kappa(rowMin))]);
        xticks(round(min(xlim)):5:round(max(xlim)))
        ytickformat('%1.3f')
%         ylim([0.999*min(ylim) max(ylim)])
    end
    if save_fig 
        print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_OptimalKappa.pdf'));
    end
return
    