%% SIMULATES A MODEL WITH TWO OBCS AND PRINTS IRFS
% 
%  OBCs - 1. Borrowing constraint; 
%         2. Macroprudential response depening on credit growth regime
%
%  Can be used to study:
%   1. an unanticipated shock
%   2. a news shock
%   3. an unrealized news shock
%
%  under the following scenarios
%   1. Symmetric rule
%   2. Asymmetric rule
%
% Uses: OCCBIN (Guerrieri and Iacoviello, 2015)
% 
% William Gatt
% 26 February 2018; 11 July 2018
%
% UPDATE FOR JMCB SUBMSSION
%  - Get updated OccBin files from Johannes Pfeifer (June 2021)
%  - Added compatability with Dynare 4.6.4
%  - Exported model equations, variable and shock declarations to a single file
% ---------------------------------------------------------------------------------

% REMEMBER: switch on control variable in Mod_borrconst_binding_sym_iterparam.mod and
% Mod_borrconst_slack_sym.mod ( iterate_params = 1 )

clear all; 
warning off

set(0,      'defaultlinelinewidth'              ,1.5)
set(groot,  'defaultAxesXGrid',                 'off')
set(groot,  'defaultAxesYGrid',                 'off')
set(groot,  'defaultAxesTickLabelInterpreter',  'latex')
set(groot,  'defaultLegendInterpreter',         'latex');
set(groot,  'defaultTextInterpreter',           'latex')

addpath('functions/toolkit_files')
addpath('functions')
save_dir = '../Figures/';
my_colours;


%% SETTINGS

% Load and/or results
    load_results = 0; %loads pre-set results (if they exist)
    save_results = 1; 
    save_fig     = 0;
    warning off
    
if load_results ==0  
% Model regimes
    regime_11       = 'Mod_borrconst_binding_sym_iterparam'; % constraint BINDS, Credit BOOM
    regime_01       = 'Mod_borrconst_slack_sym_iterparam';   % constraint SLACK, Credit BOOM

% Occasionally binding constraint
    constraint          =   'mju<-mju_ss';     % escape from reference model - constraint becomes slack
    constraint_relax    =   'MaxLev > 0';      % back to reference model     - constraint becomes binding
    
    
% Pick innovation for IRFs
    irfshock    =   char('eps_ETA','eps_ETA_surprise','eps_j','eps_j4');  % label for innovation for IRFs    
  [nshocks,yyy] =   size(irfshock);
    nshckperiods=   400; % number of periods in which shock hits (better set this low and numiter high, quicker)
    extraperiods=   100; % extra periods with shocks = 0 to allow model to converge back to SS
    numiter     =   10;  % number of times the economy is simulated (at a given parameterisation)
    shockstd    =   [0.054 0.054 0.054 0.054]; % 1 standard deviation shock --> change in q by 1%
    shock_size  =   1;
    maxiterr    =   200; % OCCBIN maximum number of iterations
    suprise     =   1.0; % 0<surprise<1, with 1 --> 100% of news shock false, 0 --> 100% of news shock true
    nperiods    =   nshckperiods + extraperiods;
    
    DynParse1   =   1; % Parse Dynare mod-files only once (be careful) (need to run short simulation once to set up structures)
    print_output=   0;


    regime_count = zeros(2,numiter);
    rng(190519,'twister'); 

    allshocks = zeros(nshckperiods,nshocks*numiter);
    convergence_errs = zeros(nshckperiods,numiter);
    

    % Main grid used
    delta_mv = [0.2:0.05:2.0]';
    rho_mv   = [0.00:0.10:0.60]';
    
    num_del = length(delta_mv);
    num_rho = length(rho_mv);
    num_del*num_rho;
    
%% SET UP SHOCK MATRICES
    switch numiter
    case 1 
        shockdraws = randn(nshckperiods,nshocks);  % number of periods x number of shocks

        % Create news shocks
        shockdraws(nshckperiods-3:nshckperiods,1) = 0;

        % Create 'surprise' (of shock not occuring)
        shockdraws(:,2) = 0;
        shockdraws(5:nshckperiods,2) = -suprise*shockdraws(1:nshckperiods-4,1);
        
        % Full matrix of shocks
        allshocks = shockdraws.*shockstd.*shock_size; 
    
    otherwise
        shockdraws = randn(nshckperiods,nshocks*numiter);
        %   Create news shocks in 'blocks' of iterations, starting from 
        %   col 1, col 4, col 7, etc (news shock, correction, preference shock)
        
        % Create 'surprise' (of shock not occuring)
        for ii = 1:nshocks:numiter*nshocks-2 
            shockdraws(nshckperiods-3:nshckperiods,ii) = 0;
            shockdraws(:,ii+1) = 0;
            shockdraws(5:nshckperiods,ii+1) = -suprise*shockdraws(1:nshckperiods-4,ii);
        end

        for iter = 1:numiter
            % Full matrix of shocks
            if nshocks>2 
                step = iter*nshocks-(nshocks-1);
                allshocks(:,step:step+nshocks-1) = shockdraws(:,step:step+nshocks-1).*shockstd.*shock_size;
            else
                step = iter*nshocks-1;
                allshocks(:,step:step+1) = shockdraws(:,step:step+1).*shockstd.*shock_size;
            end
        end
    end    


counting = 0;


%% MAIN LOOPS
    fprintf('\n\n\n=================================================== \n');
    fprintf('Solving model using OCCBIN \n - Model with OBC in borrowing \n')
    fprintf('===================================================\n');

    counter = 0;
    total_iters = num_del*num_rho;
    tic;
    
    for del = 1:num_del
        
        for rho = 1:num_rho
            
            delta_mv1 = delta_mv(del);
            rho_v1 = rho_mv(rho);
            
            fprintf('\nParameters: delta_m = %2.2f, rho_m = %2.2f\n',delta_mv1,rho_v1);
            
            %overwrite default parameters before passing them to OCCBIN
            fileid = fopen('param_calib1.mod','w');
            add_params = strcat('delta_m=',num2str(delta_mv1),'; rho_m=',num2str(rho_v1),';');
            fprintf(fileid,'%c',add_params);
            
            counter = counter + 1;
            if mod(counter,10)==0
                fprintf('--> Iteration %3.0f of %3.0f\n', counter, total_iters)
            end
            for iter = 1:numiter
                if nshocks>2
                    step = iter*nshocks-(nshocks-1);
                    shockssequence = allshocks(:,step:step+nshocks-1);
                else
                    step = iter*nshocks-1;
                    shockssequence = allshocks(:,step:step+1);
                end
                
                [zdatalinear zdatapiecewise zdatass oobase_ Mbase_ conv_errs reg_count ] = solve_one_constraint1(...
                    regime_11,regime_01,...
                    constraint,constraint_relax,...
                    shockssequence,irfshock,nperiods,maxiterr,0,DynParse1);
                
                convergence_errs(:,iter) = conv_errs;
                counting = counting + nshckperiods;
                regime_count(:,iter) = [reg_count];
                
                for i=1:Mbase_.endo_nbr
                    eval([deblank(Mbase_.endo_names{i,:}),'_l(:,iter,del,rho)   =   zdatalinear(:,i);']);
                    eval([deblank(Mbase_.endo_names{i,:}),'_p(:,iter,del,rho)   =   zdatapiecewise(:,i);']);
                    eval([deblank(Mbase_.endo_names{i,:}),'_ss                  =   zdatass(i);']);
                end
            end
            
            if sum(sum(convergence_errs)) >0
                fprintf('\nCONVERGENCE PROBLEMS in %2d simulations!\n',sum(sum(convergence_errs)));
            end
            
            mju_simulated = mju_p(1:end-extraperiods,:)+mju_ss;
            num_mju_neg = [sum(mju_simulated <= 1e-8)*100/nshckperiods]'; %number of times mju is virtually 0
            num_q_neg   = sum((sum(q_p(1:end-extraperiods)+q_ss<0)));
            num_r_neg   = sum((sum(r4_p(1:end-extraperiods)+r4_ss<0)));
            if print_output == 1
                fprintf('\n------------ OUTPUT ---------------\n')
                % fprintf('\nAv. borrowing constraint is slack: %3.1f%% \n\n', mean(num_mju_neg));
                fprintf('\nNumber of times:\n');
                fprintf('   Reference regime:               %3.1f%%\n',sum(regime_count(1,:))*100/(nshckperiods*numiter))
                fprintf('   Borrowing constraint is slack:  %3.1f%%\n',sum(regime_count(2,:))*100/(nshckperiods*numiter))
                if num_q_neg >0 || num_r_neg>0
                    fprintf('   q is negative:                  %4d\n',num_q_neg )
                    fprintf('   r is negative:                  %4d\n', num_r_neg)
                end
            end
        end
    end
    etime2=toc;
    fprintf('\nTotal simulation time: %4.4f minutes\n', etime2/60);
    
end

%% CALCULATING AND PLOTTING LOSS

    if load_results
        load("Loss_delta_rho_OBC")
    end

    Loss_credit         = NaN(num_del,num_rho);
    Loss_LTV            = NaN(num_del,num_rho);
    Loss_output         = NaN(num_del,num_rho);
    Loss                = NaN(num_del,num_rho);

    % LOSS FUNCTION
    for del = 1:num_del
        for rho = 1:num_rho
            Loss_credit(del,rho) = mean(var(B_4Y_p(1:nshckperiods,:,del,rho)));
            Loss_LTV(del,rho) = mean(var(m_p(1:nshckperiods,:,del,rho)));
            Loss_output(del,rho) = mean(var(Y_p(1:nshckperiods,:,del,rho)));
            weight_credit = 1.0;
            weight_volatility = 0.50;
            weight_output = 0;
            Loss = weight_credit*Loss_credit + weight_volatility*Loss_LTV + weight_output*Loss_output;
        end
    end
    
    minLoss = min(Loss,[],'all');
    maxLoss = max(Loss,[],'all');
    [rowMin,colMin] = find(Loss==minLoss,2);
    range = exp(linspace(log(minLoss),log(maxLoss),50));

    if save_results
        save Loss_delta_rho_OBC oo_ M_ Loss_credit Loss_LTV Loss B_4Y_p B_4Y_ss m_p m_ss Y_p Y_ss q_p q_l q_ss mju_p mju_ss delta_mv rho_mv nshckperiods extraperiods numiter shockstd shock_size num_del num_rho
    end
       
    % INTERPOLATED LOSS
    delta_mvfine = [min(delta_mv):0.025:max(delta_mv)]';
    rho_mvfine = [min(rho_mv):0.025:max(rho_mv)]';

    [r1,d1] = ndgrid(rho_mvfine, delta_mvfine);
    Loss1 = interpn(delta_mv,rho_mv,Loss,d1,r1,'spline');
    Loss_credit1 = interpn(delta_mv,rho_mv,Loss_credit,d1,r1,'spline');
    Loss_LTV1 = interpn(delta_mv,rho_mv,Loss_LTV,d1,r1,'spline');
    minLoss1 = min(Loss1,[],'all');
    maxLoss1 = max(Loss1,[],'all');
    [rowMin1,colMin1] = find(Loss1==minLoss1,2);
    
    range1 = exp(linspace(log(minLoss1),log(maxLoss1),50)); % controls countour lines in countour plot below
    fprintf('\n\nOptimal parameter values (shock std: %3.3f) \n   delta_m: %2.4f\n   rho_m  : %2.4f\n',shockstd(1)*shock_size,delta_mvfine(colMin1),rho_mvfine(rowMin1))

    
%% PLOTS FOR ONLINE APPENDIX
    
  figure('Name','Loss as a function of delta_m and rho_m','PaperPositionMode','auto',...
             'PaperOrientation','portrait',...
         'PaperSize',[20 15],'Color','white')
             
    subplot(1,2,1)
    plot(delta_mvfine,Loss_credit1(rowMin1,:),'color',red,'LineStyle','-.'); hold on; 
    plot(delta_mvfine,Loss_LTV1(rowMin1,:),'color',blue,'LineStyle','--'); 
    plot(delta_mvfine, Loss1(rowMin1,:),'color',green); hold on;
    plot(delta_mvfine(colMin1),minLoss1,'marker','o','markeredgecolor',green,'markerfacecolor',green,'markersize',5)
    xlabel('$\delta_{\mathsf{m}}$','interpreter','latex','FontSize',12);
    ax = gca;
    ytickformat('%2.1f')
    xticks(0:0.5:2)
    set(gca,'FontSize',12)

    subplot(1,2,2)
    plot(rho_mvfine,Loss_credit1(:,colMin1),'color',red,'LineStyle','-.'); hold on; 
    plot(rho_mvfine,Loss_LTV1(:,colMin1),'color',blue,'LineStyle','--'); 
    plot(rho_mvfine, Loss1(:,colMin1),'color',green)
    plot(rho_mvfine(rowMin1),minLoss1,'marker','o','markeredgecolor',green,'markerfacecolor',green,'markersize',5)

    xlabel('$\rho_{\mathsf{m}}$','interpreter','latex','FontSize',12);
    ax = gca;    
    xticks(0:0.2:0.6)
    set(gca,'FontSize',12)
        

    leg=legend('$\sigma^{2}_{\Omega}$', '$\sigma^{2}_{\mathsf{m}}$','Loss (RHS)',...
               'NumColumns',3,'FontSize',15,...
               'Position',[0.288301089672712,-0.020,0.451564487321243,0.046307],...
               'box', 'off');
           
   if save_fig 
        print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_LossDecomposition.pdf'));  
   end
    
   
   posdelta = find(delta_mvfine==0.4,1); % position in delta vector where delta is a certain number
   delta_mvfine2 = delta_mvfine(posdelta:end);
   rho_mvfine2 = rho_mvfine(1:end);
   
   Loss2 = Loss1(:,posdelta:end,:);
   d2 = d1(:,posdelta:end);
   r2 = r1(:,posdelta:end);
   minLoss2 = min(Loss2,[],'all');
   maxLoss2 = max(Loss2,[],'all');
   [rowMin2,colMin2] = find(Loss2==minLoss2,2);
   range2 = exp(linspace(log(minLoss2),log(maxLoss2),50)); % controls countour lines in countour plot below
   
   f=figure('Name','Optimal delta_m','PaperPositionMode','auto',...
       'PaperOrientation','portrait',...
       'PaperSize',[20 15],'Color','white');
   
   ax=subplot(2,2,[1; 3]);
   s = meshc(r2,d2,Loss2,'FaceAlpha','0.5'); grid on; hold on
   plot3(rho_mvfine2(rowMin2),delta_mvfine2(colMin2),minLoss2,'marker','o','markeredgecolor','r','markerfacecolor','r','markersize',7)
   xlabel('$\rho_{\mathsf{m}}$','interpreter','latex');
   ylabel('$\delta_{\mathsf{m}}$','interpreter','latex');
   zlabel('Loss');
   xlim([min(rho_mvfine2) max(rho_mvfine2)])
   xticks(min(rho_mvfine2):0.2:max(rho_mvfine2))
   yticks(0.40:0.40:max(delta_mvfine))
   ylim([min(delta_mvfine2) max(delta_mvfine2)])
   zlim([0.00065 0.0009])
   
   s(1).FaceColor = 'interp';
   view(ax,[-60.471735977184728,6.43218349182451]);
   set(gca,'FontSize',9);
   
   
   subplot(2,2,[2 4])
   contour(r2,d2,Loss2,range2,'LineStyle','-'); grid off; hold on;
   plot(rho_mvfine2(rowMin2),delta_mvfine2(colMin2),'marker','o','markeredgecolor','r','markerfacecolor','r','markersize',7)
   xlabel('$\rho_{\mathsf{m}}$','interpreter','latex');
   ylabel('$\delta_{\mathsf{m}}$','interpreter','latex');
   title('Loss')
   xticks(min(rho_mvfine2):0.1:max(rho_mvfine2));
   xlim([min(rho_mvfine2) max(rho_mvfine2)]);
   set(gca,'FontSize',9);
   
   if save_fig
       print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_OptimalDeltaSym.pdf'),'-r600');
   end


