%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
%
% Simulates model economy under two OBCs (borrowing and policy)
% William Gatt
% 
% To generate data for Fig7_8_Plot_GAR, run the code for delta_mH = 0, 0.75 and 32.5, 
% and uncomment the first the save string at the bottom (_long.mat) and comment the second string

clear all; 
clc;

addpath('functions')
addpath('functions/toolkit_files')

set(groot,'defaultlinelinewidth',1);
set(groot,'defaultAxesXGrid','on');
set(groot,'defaultAxesYGrid','on');
set(groot, 'defaultAxesTickLabelInterpreter','latex')
set(groot, 'defaultLegendInterpreter','latex');
set(groot, 'defaultTextInterpreter','latex')


% Model regimes
    Set.regime_11       = 'Mod_borrconst_binding_high_iterparam'; % constraint BINDS, Credit BOOM
    Set.regime_01       = 'Mod_borrconst_slack_high_iterparam';   % constraint SLACK, Credit BOOM
    Set.regime_10       = 'Mod_borrconst_binding_low_iterparam';  % constraint BINDS, Credit BUST
    Set.regime_00       = 'Mod_borrconst_slack_low_iterparam';    % constraint SLACK, Credit BUST

% Occasionally binding constraints
    Set.constraint1          =   'mju<-mju_ss';     % escape from reference model - constraint becomes slack
    Set.constraint_relax1    =   'MaxLev > 0';      % back to reference model     - constraint becomes binding
    Set.constraint2          =   'B_4Y < -0.0001';  % escape from reference model - constraint becomes slack
    Set.constraint_relax2    =   'B_4Y > -0.0001';  % back to reference model     - constraint becomes binding
    
% Pick innovation for IRFs
    Set.irfshock    =   char('eps_ETA','eps_ETA_surprise','eps_j','eps_j4');  % label for innovation for IRFs    
  [Set.nshocks,Set.yyy] =   size(Set.irfshock);
    Set.nshckperiods=   400; % number of periods in which shock hits (400 ==> 100 years)
    Set.extraperiods=   100; % extra periods with shocks = 0 to allow model to converge back to SS (200 seems to work and is quickest)
    Set.numiter     =   50; % for Relative Variance plots
%     Set.numiter     =   300; % for GAR plots
    Set.shockstd    =   [0.054 0.054 0.054 0.054]; % 1 standard deviation shock --> change in q by 1%
    Set.shock_size  =   1.0; 
    Set.maxiterr    =   200; % OCCBIN maximum number of iterations
    Set.suprise     =   1; % 0<surprise<1, with 1 --> 100% of news shock false, 0 --> 100% of news shock true
    Set.nperiods    =   Set.nshckperiods + Set.extraperiods;
    
    Set.DynParse1   =   1; % Parse Dynare mod-files only once (be careful) (need to run short simulation once to set up structures)
    Set.print_output=   1;
    Set.save        =   1; % Saves the results to the folder New_Results
    tic;
    
    Set.regime_count = zeros(Set.nshocks,Set.numiter);
%     tnumperiods = nshckperiods*numiter;
%     maxnumreps  = numiter;

    rng(190519,'twister');  % fixed - simulations over different policy rules need the same shocks!

    my_colours1;
   
    curdir = pwd;
    
    % uncomment whichever is needed 
    delta_m_sym = 0.75;
%     max_kink = 20;
%     delta_mHv = 0; % for checking how frequently the borr const binds when macropru is off (change parameters in param_calib)
%     delta_mHv = [0.75]'; 
%     delta_mHv = 32.5;
%     delta_mHv = linspace(delta_m_sym,delta_m_sym*max_kink,39)';
    delta_mHv = [1:0.50:35.0]';

    Set.num_del = length(delta_mHv);

    fprintf('\n======================================================== \n');
    fprintf('Solving model using OCCBIN \n - Model with OBC in borrowing and asymmetric LTV rule \n')
    fprintf('========================================================\n');

%% Set. UP SHOCK MATRICES
    
    allshocks = zeros(Set.nshckperiods,Set.nshocks*Set.numiter);
    convergence_errs = zeros(Set.nshckperiods,Set.numiter);
    
    switch Set.numiter
        case 1
            shockdraws = randn(Set.nshckperiods,Set.nshocks);  % number of periods x number of shocks
            
            % Create news shocks
            shockdraws(Set.nshckperiods-3:Set.nshckperiods,1) = 0;
            
            % Create 'surprise' (of shock not occuring)
            shockdraws(:,2) = 0;
            shockdraws(5:Set.nshckperiods,2) = -Set.suprise*shockdraws(1:Set.nshckperiods-4,1);
            
            % Full matrix of shocks
            allshocks = shockdraws.*Set.shockstd.*Set.shock_size;
            
        otherwise
            shockdraws = randn(Set.nshckperiods,Set.nshocks*Set.numiter);
            %   Create news shocks in 'blocks' of iterations, starting from
            %   col 1, col 4, col 7, etc (news shock, correction, preference shock)
            
            % Create 'surprise' (of shock not occuring)
            for ii = 1:Set.nshocks:Set.numiter*Set.nshocks-2
                shockdraws(Set.nshckperiods-3:Set.nshckperiods,ii) = 0;
                shockdraws(:,ii+1) = 0;
                shockdraws(5:Set.nshckperiods,ii+1) = -Set.suprise*shockdraws(1:Set.nshckperiods-4,ii);
            end
            
            for iter = 1:Set.numiter
                % Full matrix of shocks
                if Set.nshocks>2
                    step = iter*Set.nshocks-(Set.nshocks-1);
                    allshocks(:,step:step+Set.nshocks-1) = shockdraws(:,step:step+Set.nshocks-1).*Set.shockstd.*Set.shock_size;
                else
                    step = iter*Set.nshocks-1;
                    allshocks(:,step:step+1) = shockdraws(:,step:step+1).*Set.shockstd.*Set.shock_size;
                end
            end
    end

% Add sequence of no shocks to allow model to return to SS
%    allshocks = [allshocks; zeros(extra_periods,nshocks*numiter)];
   
% Switch off all shocks but one impluse (equivalent to short_simul=1)
%          allshocks(2:end,1) = 0;
%          allshocks(6:end,2) = 0;


%% OUTER LOOP
    for delmH = 1:Set.num_del

        %overwrite default parameter(s) before passing them to OCCBIN
        delta_mH_temp = delta_mHv(delmH);
        fileid = fopen('param_calib_sim.mod','w');
        add_params = strcat('delta_mH=',num2str(delta_mH_temp),';');
        fprintf(fileid,'%c',add_params);

        fprintf('\n----------------------------------------------\n')
        fprintf('                delta_mH = %2.2f', delta_mH_temp)
        fprintf('\n----------------------------------------------\n')
        %% PRE-PARSE MOD FILES IF NEEDED (for speed)
        if Set.DynParse1 == 1 %parse Dynare mod files once at the beginning to speed up loop execution
            fprintf('\n---------------------------------\n')
            fprintf('Pre-parsing MOD files for speed')
            fprintf('\n---------------------------------\n')

            disp('Regime 1')
            eval(['dynare ',Set.regime_11,' noclearall nolog nopreprocessoroutput'])
            oo00_ = oo_;
            M00_ = M_;
            save refmod1 oo00_ M00_

            disp('Regime 2')
            eval(['dynare ',Set.regime_01,' noclearall nopreprocessoroutput '])
            oo10_ = oo_;
            M10_ = M_;
            save refmod2 oo10_ M10_

            disp('Regime 3')
            eval(['dynare ',Set.regime_10,' noclearall nopreprocessoroutput '])
            oo01_ = oo_;
            M01_ = M_;
            save refmod3 oo01_ M01_

            disp('Regime 4')
            eval(['dynare ',Set.regime_00,' noclearall nopreprocessoroutput'])
            oo11_ = oo_;
            M11_ = M_;
            save refmod4 oo11_ M11_
            fprintf('---------------------------------\n')
%             fprintf('           ...done...');
%             fprintf('\n---------------------------------\n\n');
        end

        %% INNER LOOP

        % popup = waitbar(0,'Percent complete: 0\%   Est. time: ','Name','Running the simulation...');
        counting = 0;
        fprintf('\n Kink: %4.4f\n',delta_mH/delta_mL);
        disp(' ')
        
        for iter = 1:Set.numiter
            if Set.nshocks>2
                step = iter*Set.nshocks-(Set.nshocks-1);
                shockssequence = allshocks(:,step:step+Set.nshocks-1);
            else
                step = iter*Set.nshocks-1;
                shockssequence = allshocks(:,step:step+1);
            end
            
            if mod(iter,0.20*Set.numiter)==0
%                 fprintf('--> delta_mH = %2.2f, iteration %3.0f of %3.0f\n', delta_mH_temp, iter, Set.numiter)
                fprintf('--> iteration %3.0f of %3.0f\n',iter, Set.numiter)
            end

            [zdatalinear, zdatapiecewise, zdatass, oobase_, Mbase_, conv_errs, reg_count] = ...
                solve_two_constraints1(...
                Set.regime_11,Set.regime_01,Set.regime_10,Set.regime_00,...
                Set.constraint1, Set.constraint2,...
                Set.constraint_relax1, Set.constraint_relax2,...
                shockssequence,Set.irfshock,Set.nperiods,0,Set.maxiterr,0,Set.DynParse1);

            convergence_errs(:,iter) = conv_errs;
            counting = counting + Set.nshckperiods;
            %        regime_count(:,iter) = [reg_count(:,1); nshckperiods-sum(reg_count(:,1))];
            Set.regime_count(:,iter) = [reg_count];

            for i=1:Mbase_.endo_nbr
                eval([deblank(Mbase_.endo_names{i,:}),'_l(:,iter)   =   zdatalinear(:,i);']);
                eval([deblank(Mbase_.endo_names{i,:}),'_p(:,iter)   =   zdatapiecewise(:,i);']);
                eval([deblank(Mbase_.endo_names{i,:}),'_ss          =   zdatass(i);']);
            end
        end

        %% REPORTING RESULTS
            mju_simulated = []; % pre-allocate or clear to avoid issues
            mju_simulated = mju_p(1:end-Set.extraperiods,:)+mju_ss;
            num_mju_neg = [sum(mju_simulated <= 1e-8)*100/Set.nshckperiods]'; %number of times mju is virtually 0
            num_q_neg   = sum((sum(q_p(1:end-Set.extraperiods)+q_ss<0)));
            num_r_neg   = sum((sum(r4_p(1:end-Set.extraperiods)+r4_ss<0)));
            
%             fprintf('Simulation complete!\n');
            if Set.print_output == 1
                fprintf('\n------------ OUTPUT ---------------\n')
                fprintf('Number of times:\n');
                fprintf('   Credit boom, constraint binding:    %5.2f%%\n',sum(Set.regime_count(1,:))*100/(Set.nshckperiods*Set.numiter))
                fprintf('   Credit boom, constraint slack:      %5.2f%%\n',sum(Set.regime_count(2,:))*100/(Set.nshckperiods*Set.numiter))
                fprintf('   Credit bust, constraint binding:    %5.2f%%\n',sum(Set.regime_count(3,:))*100/(Set.nshckperiods*Set.numiter))
                fprintf('   Credit bust, constraint slack(!):   %5.2f%%\n',sum(Set.regime_count(4,:))*100/(Set.nshckperiods*Set.numiter))
                fprintf('\n Av. borrowing constraint is slack: %4.1f%% \n', mean(num_mju_neg));
            end           
            
            if sum(sum(convergence_errs)) >0
                fprintf('\nCONVERGENCE PROBLEMS in %2d simulations!\n',sum(sum(convergence_errs)));
            end
            if num_q_neg >0 || num_r_neg>0
                    fprintf('   q is negative:                  %4d\n',num_q_neg )
                    fprintf('   r is negative:                  %4d\n', num_r_neg)
            end
%             fprintf(' Number of times \n - q is negative: %4d\n - r is negative: %4d\n', sum((sum(q_p(1:end-extraperiods)+q_ss<0))),sum((sum(r4_p(1:end-extraperiods)+r4_ss<0))))
            etime=toc;     
%             fprintf('\n This simulation time: %4.4f minutes\n\n', toc/60);
        
        if Set.save
            curdir = pwd;
            cd(strcat(pwd,'/New_Results'));
         
%             save_string  = sprintf('simulation_dmH_%2.2f_long.mat',delta_mH_temp);            
            save_string  = sprintf('simulation_dmH_%2.2f.mat',delta_mH_temp);
            save(save_string, 'oo00_', 'oo01_', 'oo10_' ,'oo11_','M00_', 'M01_', 'M10_', 'M11_', ...
                'B_4Y_p', 'B_4Y_l', 'B_4Y_ss', 'Y_p', 'Y_l', 'Y_ss',...
                'PI4_p','PI4_l','PI4_ss', 'delta_mHv', 'delta_mL',...,
                'm_p','m_l','m_ss',...
                'PI_Y4_p','PI_Y4_l','PI_Y4_ss','delta_m','Set');
            cd(curdir)
        end
        
        disp(' ')
        
    end
    fprintf('\n Total simulation time: %4.4f minutes\n\n', toc/60);



